import argparse
import os
from pathlib import Path

from .defrag import Defragmenter
from .finder import (
    extract_from_js,
    extract_from_relay_files,
    extract_from_standalone_files,
)


def main():
    parser = argparse.ArgumentParser(description="Defragment GraphQL queries")
    parser.add_argument(
        "--js-dir",
        help="Directory with JavaScript files containing GraphQL fragments and queries",
    )
    parser.add_argument(
        "--graphql-dir", help="Directory with GraphQL files with .graphql extensions"
    )
    parser.add_argument(
        "--relay-dir", help="Directory with Relay files with .graphql.ts extensions"
    )
    parser.add_argument(
        "--output-dir", help="Directory to write defragmented queries to"
    )
    parser.add_argument(
        "--include-source",
        help="Include verbose source information",
        action="store_true",
        default=False,
    )
    args = parser.parse_args()

    defragmenter = Defragmenter()
    if args.js_dir:
        for doc in extract_from_js(Path(args.js_dir)):
            defragmenter.add_document(doc)
    if args.graphql_dir:
        for doc in extract_from_standalone_files(Path(args.graphql_dir)):
            defragmenter.add_document(doc)
    if args.relay_dir:
        for doc in extract_from_relay_files(Path(args.relay_dir)):
            defragmenter.add_document(doc)
    if args.output_dir:
        output_dir = Path(args.output_dir)
        os.makedirs(output_dir, exist_ok=True)
        for query_name, query in defragmenter.defragment_all(
            add_source=args.include_source
        ):
            with (output_dir / f"{query_name}.graphql").open("w") as f:
                f.write(query)


if __name__ == "__main__":
    main()



"""

Find GraphQL fragments.

"""

import ast
import os
import re
from collections.abc import Container, Iterable
from pathlib import Path


def get_files(source_dir: Path, extensions: Container[str]) -> Iterable[Path]:
    """Find all pieces of GraphQL in a directory."""
    for root, _, files in os.walk(source_dir):
        for file in files:
            path = Path(root) / file
            if path.suffix in extensions:
                yield path


def extract_from_js(source_dir: Path) -> Iterable[str]:
    """Extract GraphQL from JavaScript files."""
    for path in get_files(source_dir, {".js", ".jsx", ".ts", ".tsx"}):
        contents = path.read_text()
        for doc in re.findall(r"(?:graphql|gql)`([^`]+)`", contents):
            yield clean_gql(doc)


def extract_from_standalone_files(source_dir: Path, extension: str = ".graphql"):
    """Extract GraphQL from standalone files."""
    for path in get_files(source_dir, {extension}):
        yield path.read_text()


def extract_from_relay_files(source_dir: Path) -> Iterable[str]:
    """Extract GraphQL from Relay files."""
    for path in get_files(source_dir, {".ts"}):
        if not path.name.endswith(".graphql.ts"):
            continue
        contents = path.read_text()
        for doc in re.findall(r'"text": ("(?:\\"|[^"])+")', contents):
            doc = ast.literal_eval(doc)
            yield clean_gql(doc)


def clean_gql(gql: str) -> str:
    # Our parser doesn't like 'null', so replace it with a string
    return re.sub(r"\b(?<!\")null(?!\")\b", '"null"', gql)
